SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='STRICT_TRANS_TABLES';

DELIMITER //
DROP FUNCTION IF EXISTS str_add_glossary_words
//
CREATE FUNCTION str_add_glossary_words (p_text TEXT, p_format VARCHAR(400) )
    RETURNS TEXT
    READS SQL DATA
    /**
    * Add links to the glossary on key words
    * <br>
    * %author Ronald Speelman
    * %version 1.1
    * Example usage:

    * SELECT introtext
    *      , str_add_glossary_words(bodytext,'<a href="/glossary/{id}">{name}</a>')
    *   FROM pages
    *  WHERE id = 1;
    * See more complex examples and a description on www.moinne.com/blog/ronald
    *
    * %param p_text    String: The text that contains the keywords that must be replaced
    * %param p_format  String: A string that replaces the keyword
    *                          This string can contain three placeholders:
    *                          {id} , {name} , {description}
    * %return String
    */
    BEGIN

        DECLARE v_id INT DEFAULT NULL;
        DECLARE v_name VARCHAR(100) DEFAULT NULL;
        DECLARE v_description TEXT DEFAULT '';
        DECLARE v_replacestring VARCHAR(500) DEFAULT NULL;
        DECLARE v_result TEXT DEFAULT '';
        DECLARE v_loop_done INT DEFAULT 0;
        DECLARE cur_1 CURSOR FOR SELECT `glossary_id`,`name`,`description` FROM glossary;
        DECLARE CONTINUE HANDLER FOR NOT FOUND SET v_loop_done = 1;

        SET v_result := p_text;

        OPEN cur_1;
            REPEAT
                FETCH cur_1 INTO v_id, v_name , v_description;
                IF NOT v_loop_done THEN
                    IF (v_result REGEXP BINARY v_name) THEN
                        SET v_replacestring := REPLACE(REPLACE(REPLACE(p_format,'{name}',v_name),'{description}',v_description),'{id}',v_id);
                        SET v_result := CONCAT(LEFT(v_result,LOCATE(BINARY v_name,BINARY v_result)-1)
                                        ,v_replacestring
                                        ,SUBSTRING(v_result ,LOCATE(BINARY v_name,BINARY v_result) + CHAR_LENGTH(v_name) ,CHAR_LENGTH(v_result))
                                        );
                    END IF;
                END IF;
            UNTIL v_loop_done END REPEAT;
        CLOSE cur_1;

        RETURN v_result;

    END;
//
DELIMITER ;
SET SQL_MODE=@OLD_SQL_MODE;


